@@ -15,28 +15,33 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use super::metadata::OpendalMetaData;
+use super::utils::*;
 use dav_server::fs::{DavDirEntry, DavMetaData, FsResult};
 use futures::StreamExt;
 use futures::{FutureExt, Stream};
+use opendal::raw::normalize_path;
 use opendal::Operator;
 use opendal::{Entry, Lister};
 use std::pin::Pin;
 use std::task::Poll::Ready;
 use std::task::{ready, Context, Poll};
 
-use super::metadata::OpendalMetaData;
-use super::utils::*;
-
 /// OpendalStream is a stream of `DavDirEntry` that is used to list the contents of a directory.
 pub struct OpendalStream {
     op: Operator,
     lister: Lister,
+    path: String,
 }
 
 impl OpendalStream {
     /// Create a new opendal stream.
-    pub fn new(op: Operator, lister: Lister) -> Self {
-        OpendalStream { op, lister }
+    pub fn new(op: Operator, lister: Lister, p: &str) -> Self {
+        OpendalStream {
+            op,
+            lister,
+            path: normalize_path(p),
+        }
     }
 }
 
@@ -45,13 +50,18 @@ impl Stream for OpendalStream {
 
     fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         let dav_stream = self.get_mut();
-        match ready!(dav_stream.lister.poll_next_unpin(cx)) {
-            Some(entry) => {
-                let entry = entry.map_err(convert_error)?;
-                let webdav_entry = OpendalDirEntry::new(dav_stream.op.clone(), entry);
-                Ready(Some(Ok(Box::new(webdav_entry) as Box<dyn DavDirEntry>)))
+        loop {
+            match ready!(dav_stream.lister.poll_next_unpin(cx)) {
+                Some(entry) => {
+                    let entry = entry.map_err(convert_error)?;
+                    if entry.path() == dav_stream.path {
+                        continue;
+                    }
+                    let webdav_entry = OpendalDirEntry::new(dav_stream.op.clone(), entry);
+                    return Ready(Some(Ok(Box::new(webdav_entry) as Box<dyn DavDirEntry>)));
+                }
+                None => return Ready(None),
             }
-            None => Ready(None),
         }
     }
 }
@@ -74,7 +84,7 @@ impl DavDirEntry for OpendalDirEntry {
         self.dir_entry.name().as_bytes().to_vec()
     }
 
-    fn metadata(&self) -> dav_server::fs::FsFuture<Box<dyn DavMetaData>> {
+    fn metadata(&self) -> dav_server::fs::FsFuture<'_, Box<dyn DavMetaData>> {
         async move {
             self.op
                 .stat(self.dir_entry.path())
